Friend Class MachineDefinition
    Inherits MachineDefinitionBase
    Implements IMachineDefinition_2_1
	
	''' <summary>
	''' The version of this machine definition as shown by choosing the Version command from the Help menu of the Configure Machine Definition dialog
	''' </summary>
	Private version As System.Version =  New System.Version("1.2.5")
	
    ''' <summary>
    ''' Local variable to store the format for the number of decimal places after the decimal seperator for an X, Y and Z position.
    ''' </summary>
    Private positionFormat As String
	
    ''' <summary>
    ''' Local variable to store the format for the number of decimal places after the decimal seperator for the feedrate
    ''' </summary>
    Private feedrateFormat As String

	''' <summary>
	''' Local variable that is used to store the configuration.
	''' </summary>
	''' <remarks>Any changes made by the user from the 'Configure Machine Definition' command are assigned to this variable.</remarks>
	Private _configuration As Configuration
	
    ''' <summary>
    ''' Local variable to store the maximum feedrate.
    ''' </summary>
	Private maxFeedrate As String
		
	''' <summary>
	''' Local variable to store a collection of tool offsets
	''' </summary>
	Private toolOffsets As System.Collections.Generic.List(Of Double)
	
	''' <summary>
	''' Local variable to store a collection of tool numbers
	''' </summary>
	Private toolNumbers As System.Collections.Generic.List(Of Double)
			
	''' <summary>
	''' Local variable to store a collection of tool radiuses
	''' </summary>
	Private toolRadiuses As System.Collections.Generic.List(Of Double)
	
#Region "Events"

	''' <summary>
	''' This event handler is invoked when this Machine Definition has finished compiling.
	''' </summary>
	''' <param name="e">Provides data for the compiled event.</param>
    ''' <remarks>Here is where you should perform any one time initialization.</remarks>
	Private Sub Complied(ByVal e As CompiledEventArgs) Implements IMachineDefinition_2_1.Compiled
				
		UserConfiguredInputDialog.DeleteAllInputBoxes()
        UserConfiguredInputDialog.AddInputBox("Program Number", 50, "1")
		
	End Sub

    ''' <summary>
    ''' This event handler is invoked first and is where you configure the general settings.
    ''' </summary>
    ''' <param name="e">Provides data for the general settings event.</param>
    Private Sub GeneralSettings(ByVal e As GeneralSettingsEventArgs) Implements IMachineDefinition_2_1.GeneralSettings

        e.OutputLineNumbers = True
        e.StartLineNumber = 0
        e.LineNumberIncrement = 1
        e.ArcBreak = ArcBreak.BreakAt180Degrees
		e.NcCodeExtension = "h"
		e.ChipToChipAtcTime = _configuration.ChipToChipAtcTime
		e.RapidPositionAccelerationDeceleration = _configuration.RapidPositionAccelerationDeceleration
		e.SubRoutinesAtStart = False
		e.LineNumbering = LineNumbering.Sequentially
		e.SubRoutinesStartNumber = 1
		e.SubRoutinesNumberIncrement = 1
		
		positionFormat = "0."
		feedrateFormat = "0."
        If e.Metric Then
			For value As Integer = 0 To _configuration.NumberDecimalPlacesPositionMetric - 1
				positionFormat += "#"
			Next
			For value As Integer = 0 To _configuration.NumberDecimalPlacesFeedrateMetric - 1
				feedrateFormat += "#"
			Next
            e.MinimumArcLength = 0.0001
            e.MinimumArcRadius = 0.005
            e.MaximumArcRadius = 10000
        Else
			For value As Integer = 0 To _configuration.NumberDecimalPlacesPositionInch - 1
				positionFormat += "#"
			Next
			For value As Integer = 0 To _configuration.NumberDecimalPlacesFeedrateInch - 1
				feedrateFormat += "#"
			Next
			e.MinimumArcLength = 0.0001
            e.MinimumArcRadius = 0.0002
            e.MaximumArcRadius = 400
        	e.RapidTraverseRate = _configuration.RapidTraverseRate / 25.4
		End If

		programNumber = UserConfiguredInputDialog.GetInputBoxValue("Program Number")
		
		If _configuration.FMax
			maxFeedrate = "FMAX"		
		Else
			maxFeedrate = "F" & _configuration.MaxFeedrate		
		End If

		toolOffsets = New System.Collections.Generic.List(Of Double)
		toolNumbers = New System.Collections.Generic.List(Of Double)
		toolRadiuses = New System.Collections.Generic.List(Of Double)

		End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the file can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create the begining of the file, for example the % sign as used by a Fanuc.</remarks>
    Private Sub FileStart(ByVal e As FileStartEventArgs) Implements IMachineDefinition_2_1.FileStart
        
		Dim prgname As String
		If _configuration.UserFileNameAsProgramName Then
			prgname = e.SharpCamFilename 
		Else
			prgname = programNumber 
		End If
		prgname = prgname.ToUpper
		NcCode.LineNumber()
		If e.Metric
			NcCode.NewString("BEGIN PGM " & prgname & " MM")
		Else
			NcCode.NewString("BEGIN PGM " & prgname & " INCH")		
		End If
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the program can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create any preparatory code.</remarks>
    Private Sub ProgramStart(ByVal e As ProgramStartEventArgs) Implements IMachineDefinition_2_1.ProgramStart

		If _configuration.OutputCycleTime Then
       	 	NcCode.LineNumber()
			NcCode.NewString(";CYCLE TIME " & New System.TimeSpan( 0, 0, e.CycleTime).TotalMinutes.ToString("0.00") & " MINUTES")
			NcCode.NewLine()
		End If
	
		If _configuration.OutputToolDef
			For Each toolNumber As Double In e.ToolNumbers
				NcCode.LineNumber()
				NcCode.NewString("TOOL DEF " & toolNumber & " L0 R0")
				NcCode.NewLine()
			Next
		End If
	
		'List all tools used in this program
		If _configuration.ToolList Then
       		If e.ToolNames.Count > 0 Then
         	   	For Each name As String In e.ToolNames
        	        NcCode.LineNumber()
        	        NcCode.NewString(";" & name.ToUpper())
        	        NcCode.NewLine()
        	    Next
       	 	End If
  		End If
 
		If _configuration.OutputBlankForm Then
			NcCode.LineNumber()
        	NcCode.NewString("BLK FORM 0.1 Z X" & e.BlankForm.MinimumCornerX.ToString(positionFormat, CultureInfo) _
												  & " Y"  & e.BlankForm.MinimumCornerY.ToString(positionFormat, CultureInfo) _
												  & " Z" & e.BlankForm.MinimumCornerZ.ToString(positionFormat, CultureInfo))
        	NcCode.NewLine()
        	NcCode.LineNumber()
        	NcCode.NewString("BLK FORM 0.2 X" & e.BlankForm.MaximumCornerX.ToString(positionFormat, CultureInfo) _
									    	   & " Y"  & e.BlankForm.MaximumCornerY.ToString(positionFormat, CultureInfo) _
											   & " Z" & e.BlankForm.MaximumCornerZ.ToString(positionFormat, CultureInfo))
            NcCode.NewLine()
		End If

        Me.ZAxisRetract(e.Metric)
		
		If _configuration.OutputM01ForATC = M01Type.OutputBeforeATC Then
        	NcCode.LineNumber()
        	NcCode.NewString("M01")
			NcCode.NewLine()
		End If

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the tool change event.</param>
    Private Sub ToolChange(ByVal e As ToolChangeEventArgs) Implements IMachineDefinition_2_1.ToolChange
        If _configuration.StructuredBlockAtATC Then
			NcCode.LineNumber()
        	NcCode.NewString("* -  " & e.ToolName)
        	NcCode.NewLine()
		End If
				
        NcCode.LineNumber()
        NcCode.NewString("TOOL CALL " & e.ToolNumber & " Z S" & e.SpindleSpeed)
		If _configuration.ToolList Then
			NcCode.NewString(" ;LOAD - " & e.ToolName)
		End If
        NcCode.NewLine()
 
		If _configuration.PreselectNextTool Then
        	NcCode.LineNumber()
        	NcCode.NewString("TOOL DEF " & e.NextToolNumber)
			If _configuration.ToolList Then
        		NcCode.NewString(";PRE SELECT - " & e.NextToolName)
			End If
        	NcCode.NewLine()
		End If

		NcCode.NewLine()
		If _configuration.M6RequiredForATC Then
        	NcCode.LineNumber()
        	NcCode.NewString("M6")
        	NcCode.NewLine()
		End If
	
		If _configuration.OutputM01ForATC = M01Type.OutputAfterATC Then
        	NcCode.LineNumber()
        	NcCode.NewString("M01")
			NcCode.NewLine()
		End If
    		
		toolOffsets.Add(e.ToolOffsetNumber)
    	toolNumbers.Add(e.ToolNumber)	
		toolRadiuses.Add(e.ToolDiameter / 2)

	End Sub

    ''' <summary>
    ''' This event handler is invoked before every tool change, unless it is the first, where it is not called.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel tool change event.</param>
    Private Sub CancelToolChange(ByVal e As CancelToolChangeEventArgs) Implements IMachineDefinition_2_1.CancelToolChange

        Me.ZAxisRetract(e.Metric)
					
		If _configuration.OutputM01ForATC = M01Type.OutputBeforeATC
        	NcCode.LineNumber()
        	NcCode.NewString("M01")
			NcCode.NewLine()
		End If
	
    End Sub

    ''' <summary>
    ''' This event handler is invoked for the first position after a tool change for Profiling and Pocketing operations.
    ''' </summary>
    ''' <param name="e">Provides data for the first position after tool change event.</param>
    Private Sub FirstPositionAfterToolChange(ByVal e As FirstPositionAfterToolChangeEventArgs) Implements IMachineDefinition_2_1.FirstPositionAfterToolChange

        NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(positionFormat, CultureInfo), Modal.Off)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(positionFormat, CultureInfo), Modal.Off)
		If _configuration.UseM13 And e.CoolantType = Coolant.Flood And e.SpindleDirection = SpindleDirection.CW Then
			NcCode.NewString("R0 " & maxFeedrate & " M13")
		Else If _configuration.UseM13 And e.CoolantType = Coolant.Flood And e.SpindleDirection = SpindleDirection.CCW Then
			NcCode.NewString("R0 " & maxFeedrate & " M14")
		Else
			NcCode.NewString("R0 " & maxFeedrate  & " " & e.ToolDirection)
		End If
        NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("L ")
        NcCode.NewDimensionedWord("Z", e.PositionZ.ToString(positionFormat, CultureInfo), Modal.Off)
		If _configuration.UseM13 And e.CoolantType = Coolant.Flood
        	NcCode.NewString("R0 " & maxFeedrate)
		Else
        	NcCode.NewString("R0 " & maxFeedrate & " " & e.Coolant)
		End If
        NcCode.NewLine()


    End Sub

    ''' <summary>
    ''' This event handler is invoked for all Profiling and Pocketing rapid positions, other than
    ''' the <see cref="FirstPositionAfterToolChange"/>.
    ''' </summary>
    ''' <param name="e">Provides data for the rapid position event.</param>
    Private Sub RapidPositiong(ByVal e As RapidPositionEventArgs) Implements IMachineDefinition_2_1.RapidPositioning

        NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewString("R0 " & maxFeedrate & " ")
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every linear interpolation move.
    ''' </summary>
    ''' <param name="e">Provides data for the linear interpolation event.</param>
    Private Sub LinearInterpolation(ByVal e As LinearInterpEventArgs) Implements IMachineDefinition_2_1.LinearInterpolation

        NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(positionFormat, CultureInfo), Modal.On)
		If _configuration.NonModalFeedrate Then
			NcCode.NewString("F" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & " M")
		Else
			NcCode.NewDimensionedWord("F", e.FeedRate.ToString(feedrateFormat, CultureInfo), Modal.On)
		End If
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is applied on the left.
    ''' </summary>
    ''' <param name="e">Provides data for the apply CRC left event</param>
    ''' <remarks>This is a linear move. CRC is not applied on circular interpolation moves</remarks>
    Private Sub ApplyCRCLeft(ByVal e As ApplyCRCLeftEventArgs) Implements IMachineDefinition_2_1.ApplyCRCLeft

        NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewWord("RL ", Modal.Off)
		If _configuration.NonModalFeedrate Then
			NcCode.NewString("F" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & " M")
		Else
        	NcCode.NewDimensionedWord("F", e.FeedRate.ToString(feedrateFormat, CultureInfo), Modal.On)
		End If
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is applied on the right.
    ''' </summary>
    ''' <param name="e">Provides data for the apply CRC right event</param>
    ''' <remarks>This is a linear move. CRC is not applied on circular interpolation moves</remarks>
    Private Sub ApplyCRCRight(ByVal e As ApplyCRCRightEventArgs) Implements IMachineDefinition_2_1.ApplyCRCRight

        NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewString("RR ")
		If _configuration.NonModalFeedrate Then
			NcCode.NewString("F" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & " M")
		Else
	        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(feedrateFormat, CultureInfo), Modal.On)
		End If
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is canceled.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel CRC right event.</param>
    ''' <remarks>This is a linear move. CRC is not cancelled on circular interpolation moves.</remarks>
    Private Sub CancelCRC(ByVal e As CancelCRCEventArgs) Implements IMachineDefinition_2_1.CancelCRC

        NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.AbsX.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsY.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewString("R0")
		If _configuration.NonModalFeedrate Then
			NcCode.NewString("F" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & " M")
		Else
 	       NcCode.NewDimensionedWord("F", e.FeedRate.ToString(feedrateFormat, CultureInfo), Modal.On)
		End If		
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every counter clockwise circular interpolation move.
    ''' </summary>
    ''' <param name="e">Provides data for the clockwise circular interpolation event</param>
    Private Sub CWInterpolation(ByVal e As CWInterpolationEventArgs) Implements IMachineDefinition_2_1.CWInterpolation

        NcCode.LineNumber()
        NcCode.NewString("CC X" & e.AbsCentreX.ToString(positionFormat, CultureInfo) & " Y" & e.AbsCentreY.ToString(positionFormat, CultureInfo))
        NcCode.NewLine()
        NcCode.LineNumber()
		If e.Helical Then
			NcCode.NewString("CP PA " & e.EndPointAngle.ToString(positionFormat, CultureInfo) & " ")
			NcCode.UpdateDimensionedWord("X", e.AbsEndX.ToString(positionFormat, CultureInfo))
			NcCode.UpdateDimensionedWord("Y", e.AbsEndY.ToString(positionFormat, CultureInfo))
			NcCode.NewDimensionedWord("Z", e.AbsEndZ.ToString(positionFormat, CultureInfo), Modal.On)
			NcCode.NewString("DR- ")
		Else
			NcCode.NewString("C ")
			NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(positionFormat, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(positionFormat, CultureInfo), Modal.On)
			NcCode.NewString("DR- ")
       	End If
		If _configuration.NonModalFeedrate Then
			NcCode.NewString("F" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & " M")
		Else
			NcCode.NewDimensionedWord("F", e.FeedRate.ToString(feedrateFormat, CultureInfo), Modal.On)
		End If
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every clockwise circular interpolation move
    ''' </summary>
    ''' <param name="e">Provides data for the circular interpolation event.</param>
    Private Sub CCWInterpolation(ByVal e As CCWInterpolationEventArgs) Implements IMachineDefinition_2_1.CCWInterpolation

        NcCode.LineNumber()
        NcCode.NewString("CC X" & e.AbsCentreX.ToString(positionFormat, CultureInfo) & " Y" & e.AbsCentreY.ToString(positionFormat, CultureInfo))
        NcCode.NewLine()
        NcCode.LineNumber()
		If e.Helical Then
			NcCode.NewString("CP PA " & e.EndPointAngle.ToString(positionFormat, CultureInfo) & " ")
			NcCode.UpdateDimensionedWord("X", e.AbsEndX.ToString(positionFormat, CultureInfo))
			NcCode.UpdateDimensionedWord("Y", e.AbsEndY.ToString(positionFormat, CultureInfo))
			NcCode.NewDimensionedWord("Z", e.AbsEndZ.ToString(positionFormat, CultureInfo), Modal.On)
			NcCode.NewString("DR+ ")
		Else
			NcCode.NewString("C ")
			NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(positionFormat, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(positionFormat, CultureInfo), Modal.On)
			NcCode.NewString("DR+ ")
		End If
		If _configuration.NonModalFeedrate Then
			NcCode.NewString("F" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & " M")
		Else
        	NcCode.NewDimensionedWord("F", e.FeedRate.ToString(feedrateFormat, CultureInfo), Modal.On)
		End If
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill the first hole after a tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the drill first hole after toolchange event</param>
    Private Sub DrillFirstHoleAfterToolChange(ByVal e As DrillFirstHoleAfterToolChangeEventArgs) Implements IMachineDefinition_2_1.DrillFirstHoleAfterToolChange

		Me.DrillFirstHole(e, True)
		
    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill the first hole in an operation that is NOT after a Tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the drill first hole in operation event.</param>
    Private Sub DrillFirstHoleInOperation(ByVal e As DrillFirstHoleInOperationEventArgs) Implements IMachineDefinition_2_1.DrillFirstHoleInOperation

		Me.DrillFirstHole(New DrillFirstHoleAfterToolChangeEventArgs(e.PositionX, e.PositionY, e.DrillingType, _
																	 String.Empty, New ToolType(), 0, 0, _
																	 String.Empty, 0, String.Empty, 0 , _
																	 e.Coolant, 0, String.Empty, _
																	 e.FeedRate, e.SpindleSpeed, _
																	 e.IntialRapid, e.FeedFrom, _
																	 e.MaterialSurface, e.FinishDepth, _
																	 e.PeckAmount, e.Dwell, e.Pitch, _
																	 e.RetractToIntialRapid, e.CoolantType, _
																	 e.SpindleDirection, e.Metric), False)

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill a hole that is NOT after a Tool change or the first in the operation.
    ''' </summary>
    ''' <param name="e">Provides data for the drill hole event.</param>
    Private Sub DrillHole(ByVal e As DrillHoleEventArgs) Implements IMachineDefinition_2_1.DrillHole

        NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(positionFormat, CultureInfo), Modal.On)
        NcCode.NewString("R0 " & maxFeedrate & " M99")
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when a drilling cycle needs to be canelled.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel drilling cycle event.</param>
	Private Sub CancelDrillingCycles(ByVal e As CancelDrillingCyclesEventArgs) Implements IMachineDefinition_2_1.CancelDrillingCycles

    End Sub

    ''' <summary>
    ''' This event handler is invoked last.
    ''' </summary>
    Private Sub ProgramEnd(ByVal e As ProgramEndEventArgs) Implements IMachineDefinition_2_1.ProgramEnd

		Dim unloadYPosition, zAxisPosition As String
		If e.Metric
			zAxisPosition =  _configuration.ZAxisToolChangePosition.ToString(positionFormat, CultureInfo)
			unloadYPosition =  _configuration.YAxisUnloadPosition.ToString(positionFormat, CultureInfo)
		Else
			zAxisPosition = (_configuration.ZAxisToolChangePosition / 25.4).ToString(positionFormat, CultureInfo)
			unloadYPosition = (_configuration.YAxisUnloadPosition / 25.4).ToString(positionFormat, CultureInfo)
		End If

		If _configuration.YAxisUnloadMethod = YAxisUnloadMethod.ToolCall0
        	NcCode.LineNumber()
			NcCode.NewString("TOOL CALL 0")
			NcCode.NewLine()
        	NcCode.LineNumber()
		    NcCode.NewString("L Z" & zAxisPosition & " R0 " & maxFeedrate)
			NcCode.NewLine()
        	NcCode.LineNumber()
		    NcCode.NewString("L Y" & unloadYPosition & " R0 " & maxFeedrate)
			NcCode.NewLine()
		Else If _configuration.YAxisUnloadMethod = YAxisUnloadMethod.ZM91
		    Me.ZAxisRetract(e.Metric)
        	NcCode.LineNumber()
		    NcCode.NewString("L Y" & unloadYPosition & " R0 " & maxFeedrate & " M91")
			NcCode.NewLine()
		Else If _configuration.YAxisUnloadMethod = YAxisUnloadMethod.ZM92
		    Me.ZAxisRetract(e.Metric)
        	NcCode.LineNumber()
		    NcCode.NewString("L Y" & unloadYPosition & " R0 " & maxFeedrate & " M92")
			NcCode.NewLine()
		End If
		
		If _configuration.ProgramEndMCode = ProgramEndMCode.M30 Then
			NcCode.LineNumber()
			NcCode.NewString("M30")
        	NcCode.NewLine()
		Else If _configuration.ProgramEndMCode = ProgramEndMCode.M02 Then
			NcCode.LineNumber()
			NcCode.NewString("M02")
        	NcCode.NewLine()
		End If

    End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the file can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create the end of the file, for example the % sign as used by a Fanuc.</remarks>
    Private Sub FileEnd(ByVal e As FileEndEventArgs) Implements IMachineDefinition_2_1.FileEnd        

		NcCode.LineNumber()
		Dim prgname As String
		If _configuration.UserFileNameAsProgramName Then
			prgname = e.SharpCamFilename 
		Else
			prgname = programNumber 
		End If
		prgname = prgname.ToUpper
		If e.Metric
			NcCode.NewString("END PGM " & prgname & " MM")
		Else
			NcCode.NewString("END PGM " & prgname & " INCH")		
		End If
        NcCode.NewLine()

		If _configuration.OutputToolOffsetValue Then
			For index As Integer = 0 To toolOffsets.Count - 1
				For Each line As System.Text.StringBuilder In e.NcCodeLines
					line.Replace("TOOL DEF " & toolNumbers(index) & " L0 R0", _
								"TOOL DEF " & toolNumbers(index) & " L" & toolOffsets(index).ToString(positionFormat, CultureInfo) & _
								" R" & toolRadiuses(index).ToString(positionFormat, CultureInfo))
				Next
			Next index
		End If

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the start of all Sub Routines.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine start event.</param>
    Private Sub SubRoutineAllStart(ByVal e As SubRoutineAllStartEventArgs) Implements IMachineDefinition_2_1.SubRoutineAllStart

	End Sub

    ''' <summary>
    ''' This event handler is invoked when a Sub Routine is called.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine start event.</param>
    Private Sub SubRoutineCall(ByVal e As SubRoutineCallEventArgs) Implements IMachineDefinition_2_1.SubRoutineCall

		NcCode.LineNumber()
        NcCode.NewString("CALL LBL" & e.SubRoutineNumber)
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the start of the Sub Routine.
    ''' </summary>
    ''' <param name="e">Provides data for the sub routine start event.</param>
    Private Sub SubRoutineStart(ByVal e As SubRoutineStartEventArgs) Implements IMachineDefinition_2_1.SubRoutineStart
        
		NcCode.LineNumber()
		NcCode.NewString("LBL" & e.SubRoutineNumber)
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the end of the Sub Routine.
    ''' </summary>
    ''' <param name="e">Provides data for the sub routine end event.</param>
    Private Sub SubRoutineEnd(ByVal e As SubRoutineEndEventArgs) Implements IMachineDefinition_2_1.SubRoutineEnd

		NcCode.LineNumber()
		NcCode.NewString("LBL0")
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the end of all Sub Routines.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine end event.</param>
    Private Sub SubRoutineAllEnd(ByVal e As SubRoutineAllEndEventArgs) Implements IMachineDefinition_2_1.SubRoutineAllEnd

	End Sub

	''' <summary>
	''' <summary>
	''' This event handler is invoked so a program stop/comment can be created.
	''' </summary>
	''' <param name="e">Provides data for the program stop event.</param>
	''' <remarks>Here is where you should create all the logic for a program stop, typlically M00/M01.</remarks>
	Private Sub ProgramStop(ByVal e As ProgramStopEventArgs) Implements IMachineDefinition_2_1.ProgramStop

		If e.ProgramStopType = ProgramStopType.Stop Then
        	NcCode.LineNumber()
			NcCode.NewString("M00")
			NcCode.NewLine()			
		Else If e.ProgramStopType = ProgramStopType.Optional Then
        	NcCode.LineNumber()
			NcCode.NewString("M01")
			NcCode.NewLine()			
		End If
		For Each commentLine As String In  e.Comment
                NcCode.LineNumber()
				If commentLine.Length > 0 Then
                	NcCode.NewString(";" & commentLine.ToUpper())				
				End If
                NcCode.NewLine()
        Next
	
	End Sub

	''' <summary>
	''' This event handler is invoked at the beginning of an interpolation move, for Profiling Operations only.
	''' </summary>
	''' <param name="e">Provides data for the start interpolation event.</param>
	''' <remarks>Used for turning on the torch when using Jet cutting.</remarks>
	Private Sub StartInterpolation(ByVal e As StartInterpolationEventArgs) Implements IMachineDefinition_2_1.StartInterpolation

	End Sub

	''' <summary>
	''' This event handler is invoked at the end of an interpolation move, for Profiling Operations only.
	''' </summary>
	''' <param name="e">Provides data for the end interpolation event.</param>
	''' <remarks>Used for turning off the torch when using Jet cutting.</remarks>
	Private Sub EndInterpolation(ByVal e As EndInterpolationEventArgs) Implements IMachineDefinition_2_1.EndInterpolation

	End Sub

#End Region

#Region "Drilling Cyle Subs"

	''' <summary>
	''' Drill the first hole after a tool change or in operation
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub DrillFirstHole(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)
	
		If e.DrillingType = DrillingType.Drilling
			If _configuration.DrillingCycle = DrillingCycle.Cycle1 Then
				Cycle1(e, True, afterToolChange)
			Else If _configuration.DrillingCycle = DrillingCycle.Cycle200
				Cycle200(e, True, afterToolChange)
			Else If _configuration.DrillingCycle = DrillingCycle.Cycle203
				Cycle203(e, True, False, afterToolChange)
			End If
		Else If e.DrillingType = DrillingType.PeckDrill	
			If _configuration.PeckDrillCycle= PeckDrillCycle.Cycle1 Then
				Cycle1(e, False, afterToolChange)
			Else If _configuration.PeckDrillCycle = PeckDrillCycle.Cycle200
				Cycle200(e, False, afterToolChange)
			Else If _configuration.PeckDrillCycle = PeckDrillCycle.Cycle203
				Cycle203(e, False, False, afterToolChange)
			Else If _configuration.PeckDrillCycle = PeckDrillCycle.Cycle205
				Cycle205(e, False, False, afterToolChange)
			End If
		Else If e.DrillingType = DrillingType.ChipBreaker	
			If _configuration.ChipBreakerCycle = ChipBreakerCycle.Cycle203
				Cycle203(e, False, True, afterToolChange)
			Else If _configuration.ChipBreakerCycle = ChipBreakerCycle.Cycle205
				Cycle205(e, False, True, afterToolChange)
			End If
		Else If e.DrillingType = DrillingType.Tapping	
			If _configuration.TappingCycle = TappingCycle.Cycle2
				Cycle2(e, afterToolChange)
			Else If _configuration.TappingCycle = TappingCycle.Cycle206
				Cycle206(e, afterToolChange)
			End If
		Else If e.DrillingType = DrillingType.RigidTapping	
			If _configuration.RigidTappingCycle = RigidTappingCycle.Cycle17
				Cycle17(e, afterToolChange)
			Else If _configuration.RigidTappingCycle = RigidTappingCycle.Cycle207
				Cycle207(e, afterToolChange)
			End If
		Else If e.DrillingType = DrillingType.Reaming	
			Cycle201(e, afterToolChange)
		Else If e.DrillingType = DrillingType.Boring	
			Cycle202(e, afterToolChange)
        End If	
        NcCode.NewLine()
		
	End Sub

	''' <summary>
	''' Output X, Y, Z position for old style cycles
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub DrillOldCycle(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)
		
	    NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(positionFormat, CultureInfo), Modal.Off)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(positionFormat, CultureInfo), Modal.Off)
		If afterToolChange Then
			If _configuration.UseM13 And e.CoolantType = Coolant.Flood And e.SpindleDirection = SpindleDirection.CW Then
				NcCode.NewString("R0 " & maxFeedrate & " M13")
			Else If _configuration.UseM13 And e.CoolantType = Coolant.Flood And e.SpindleDirection = SpindleDirection.CCW Then
				NcCode.NewString("R0 " & maxFeedrate & " M14")
			Else
				NcCode.NewString("R0 " & maxFeedrate  & " " & e.ToolDirection)
			End If
			NcCode.NewLine()
			NcCode.LineNumber()

			NcCode.NewString("L Z" & e.IntialRapid.ToString(positionFormat, CultureInfo) & " R0 " & maxFeedrate & " ")
			If Not _configuration.UseM13 Or Not e.CoolantType = Coolant.Flood Then
	        	NcCode.NewString(e.Coolant)
			End If
			NcCode.NewLine()
			
			NcCode.LineNumber()
			NcCode.NewString("L Z" & e.FeedFrom.ToString(positionFormat, CultureInfo) & " R0 " & maxFeedrate & " M99")
        Else
			NcCode.NewString(" Z" & e.FeedFrom.ToString(positionFormat, CultureInfo) & " R0 " & maxFeedrate & " M99")
		End If
		NcCode.NewLine()

	End Sub

	''' <summary>
	''' Output X, Y position for new style cycles
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub DrillNewCycle(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)
		
        NcCode.LineNumber()
        NcCode.NewString("L ")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(positionFormat, CultureInfo), Modal.Off)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(positionFormat, CultureInfo), Modal.Off)
		If afterToolChange Then
			If _configuration.UseM13 And e.CoolantType = Coolant.Flood And e.SpindleDirection = SpindleDirection.CW Then
				NcCode.NewString("R0 " & maxFeedrate & " M13")
			Else If _configuration.UseM13 And e.CoolantType = Coolant.Flood And e.SpindleDirection = SpindleDirection.CCW Then
				NcCode.NewString("R0 " & maxFeedrate & " M14")
			Else
				NcCode.NewString("R0 " & maxFeedrate  & " " & e.ToolDirection)
			End If
			NcCode.NewLine()
			NcCode.LineNumber()

			NcCode.NewString("L Z" & e.IntialRapid.ToString(positionFormat, CultureInfo) & " R0 " & maxFeedrate & " ")
			If Not _configuration.UseM13 Or Not e.CoolantType = Coolant.Flood Then
	        	NcCode.NewString(e.Coolant)
			End If
			NcCode.NewLine()

			NcCode.LineNumber()
			NcCode.NewString("CYCL CALL")
		Else
			NcCode.NewString(" R0 " & maxFeedrate & " M99")
		End If
        NcCode.NewLine()

	End Sub

	''' <summary>
	''' Output code for Cycle 1
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="noPeck">A value indicating if the cycle is to drill to full depth and not peck. <c>True</c> for drill to full depth, <c>false</c> to peck</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle1(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, _
					   ByVal noPeck As Boolean, _
					   ByVal afterToolChange As Boolean)

	    NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 1.0 PECKING")
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 1.1 SET UP " &  (e.MaterialSurface - e.FeedFrom).ToString(positionFormat, CultureInfo))
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 1.2 DEPTH " & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo))
		NcCode.NewLine()
        NcCode.LineNumber()
		If noPeck
			NcCode.NewString("CYCL DEF 1.3 PECKG " & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo))
		Else
			NcCode.NewString("CYCL DEF 1.3 PECKG " & -e.PeckAmount.ToString(positionFormat, CultureInfo))
		End If
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 1.4 DWELL " & e.Dwell.ToString(positionFormat, CultureInfo))
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 1.5 F" & e.FeedRate.ToString(feedrateFormat, CultureInfo))
		NcCode.NewLine()
		
		Me.DrillOldCycle(e, afterToolChange)
		
	End Sub

	''' <summary>
	''' Output code for Cycle 2
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle2(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)

		NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 2.0 TAPPING")
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 2.1 SET UP " &  (e.MaterialSurface - e.FeedFrom).ToString(positionFormat, CultureInfo))
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 2.2 DEPTH " & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo))
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 2.3 DWELL " & e.Dwell.ToString(positionFormat, CultureInfo))
		NcCode.NewLine()
        NcCode.LineNumber()
		'Reduce feedrate by 5% for floating holder
		NcCode.NewString("CYCL DEF 2.4 F" & (e.Pitch * e.SpindleSpeed * 0.95).ToString(feedrateFormat, CultureInfo)) 
		NcCode.NewLine()
		
		Me.DrillOldCycle(e, afterToolChange)
		
	End Sub

	''' <summary>
	''' Output code for Cycle 17
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle17(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)

		NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 17.0 RIGID TAPPING GS")
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 17.1 SET UP " & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo))
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 17.2 DEPTH " & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo))
		NcCode.NewLine()
        NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 17.3 F" & e.Pitch.ToString(positionFormat, CultureInfo)) 
		NcCode.NewLine()
		
		Me.DrillOldCycle(e, afterToolChange)
		
	End Sub

	''' <summary>
	''' Output code for Cycle 200
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="noPeck">A value indicating if the cycle is to drill to full depth and not peck. <c>True</c> for drill to full depth, <c>false</c> to peck</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle200(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, _
						 ByVal noPeck As Boolean, _
						 ByVal afterToolChange As Boolean)
	
	    NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 200 DRILLING ~")
		NcCode.NewLine()
		NcCode.NewString("  Q200=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";SET-UP CLEARANCE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q201=" & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo)  & ";DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q206=" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & ";FEED RATE FOR PLNGNG ~")
		NcCode.NewLine()
		If noPeck
			NcCode.NewString("  Q202=" &  (e.MaterialSurface - e.FinishDepth).ToString(positionFormat, CultureInfo) & ";PLUNGING DEPTH ~")
		Else
			NcCode.NewString("  Q202=" & e.PeckAmount.ToString(positionFormat, CultureInfo) & ";PLUNGING DEPTH ~")
		End If
		NcCode.NewLine()
		NcCode.NewString("  Q210=0;DWELL TIME AT TOP ~")
		NcCode.NewLine()
		NcCode.NewString("  Q203=" & e.MaterialSurface.ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		If e.RetractToIntialRapid
			NcCode.NewString("  Q204=" & (e.IntialRapid - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE ~")
		Else
			NcCode.NewString("  Q204=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE ~")
		End If
		NcCode.NewLine()
		NcCode.NewString("  Q211=" & e.Dwell.ToString(positionFormat, CultureInfo) & ";DWELL TIME AT DEPTH")
		NcCode.NewLine()
		
		Me.DrillNewCycle(e, afterToolChange)
				
	End Sub

	''' <summary>
	''' Output code for Cycle 201
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle201(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)
	
	    NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 201 REAMING ~")
		NcCode.NewLine()
		NcCode.NewString("  Q200=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";SET-UP CLEARANCE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q201=" & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo)  & ";DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q206=" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & ";FEED RATE FOR PLNGNG ~")
		NcCode.NewLine()
		NcCode.NewString("  Q211=" & e.Dwell.ToString(positionFormat, CultureInfo) & ";DWELL TIME AT DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q208=0;RETRACTION FEED RATE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q203=" & e.MaterialSurface.ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		If e.RetractToIntialRapid
			NcCode.NewString("  Q204=" & (e.IntialRapid - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE")
		Else
			NcCode.NewString("  Q204=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE")
		End If
		NcCode.NewLine()
		
		Me.DrillNewCycle(e, afterToolChange)
		
	End Sub

	''' <summary>
	''' Output code for Cycle 202
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle202(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)
	
	    NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 202 BORING ~")
		NcCode.NewLine()
		NcCode.NewString("  Q200=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";SET-UP CLEARANCE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q201=" & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo)  & ";DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q206=" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & ";FEED RATE FOR PLNGNG ~")
		NcCode.NewLine()
		NcCode.NewString("  Q211=" & e.Dwell.ToString(positionFormat, CultureInfo) & ";DWELL TIME AT DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q208=0;RETRACTION FEED RATE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q203=" & e.MaterialSurface.ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		If e.RetractToIntialRapid
			NcCode.NewString("  Q204=" & (e.IntialRapid - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE ~")
		Else
			NcCode.NewString("  Q204=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE ~")
		End If
		NcCode.NewLine()
		NcCode.NewString("  Q214=0;DISENGAGING DIRECTION ~")
		NcCode.NewLine()
		NcCode.NewString("  Q336=0;ANGLE OF SPINDLE")
		NcCode.NewLine()
		
		Me.DrillNewCycle(e, afterToolChange)
		
	End Sub

	''' <summary>
	''' Output code for Cycle 203
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="noPeck">A value indicating if the cycle is to drill to full depth and not peck. <c>True</c> for drill to full depth, <c>false</c> to peck</param>
	''' <param name="retractFeedRate">Feedrate for parameter 208</param>
	''' <param name="chipBreak">A value indicating if the cycle should chipbreak. <c>True</c> for chipbreak, <c>false</c> for pecking with full retract</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle203(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, _
						ByVal noPeck As Boolean, _
						ByVal chipBreak As Boolean, _
						ByVal afterToolChange As Boolean)
	
	    NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 203 UNIVERSAL DRILLING ~")
		NcCode.NewLine()
		NcCode.NewString("  Q200=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";SET-UP CLEARANCE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q201=" & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo)  & ";DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q206=" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & ";FEED RATE FOR PLNGNG ~")
		NcCode.NewLine()
		If noPeck 
			NcCode.NewString("  Q202=" & (e.MaterialSurface - e.FinishDepth).ToString(positionFormat, CultureInfo) & ";PLUNGING DEPTH ~")
		Else
			NcCode.NewString("  Q202=" & e.PeckAmount.ToString(positionFormat, CultureInfo) & ";PLUNGING DEPTH ~")
		End If
		NcCode.NewLine()
		NcCode.NewString("  Q210=0;DWELL TIME AT TOP ~")
		NcCode.NewLine()
		NcCode.NewString("  Q203=" & e.MaterialSurface.ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		If e.RetractToIntialRapid
			NcCode.NewString("  Q204=" & (e.IntialRapid - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE ~")
		Else
			NcCode.NewString("  Q204=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE ~")
		End If
		NcCode.NewLine()
		NcCode.NewString("  Q212=0;DECREMENT ~")
		NcCode.NewLine()
		If chipBreak
			NcCode.NewString("  Q213=9999;BREAKS ~")
		Else
			NcCode.NewString("  Q213=1;BREAKS ~")
		End If
		NcCode.NewLine()
		NcCode.NewString("  Q205=0;MIN. INFEED DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q211=" & e.Dwell.ToString(positionFormat, CultureInfo) & ";DWELL TIME AT DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q208=" & _configuration.MaxFeedrate.ToString(positionFormat, CultureInfo) &";RETRACTION FEEDRATE ~")
		NcCode.NewLine()
		'Retract by the effective pitch for chip break
		NcCode.NewString("  Q256=" & (e.FeedRate/e.SpindleSpeed).ToString(feedrateFormat, CultureInfo) & ";DIST. FOR CHIP BRKNG")
		NcCode.NewLine()
		
		Me.DrillNewCycle(e, afterToolChange)
		
	End Sub

	''' <summary>
	''' Output code for Cycle 205
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="noPeck">A value indicating if the cycle is to drill to full depth and not peck. <c>True</c> for drill to full depth, <c>false</c> to peck</param>
	''' <param name="retractFeedRate">Feedrate for parameter 208</param>
	''' <param name="chipBreak">A value indicating if the cycle should chipbreak. <c>True</c> for chipbreak, <c>false</c> for pecking with full retract</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle205(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, _
						ByVal noPeck As Boolean, _
						ByVal chipBreak As Boolean, _
						ByVal afterToolChange As Boolean)
	
	    NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 205 UNIVERSAL PECKING ~")
		NcCode.NewLine()
		NcCode.NewString("  Q200=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";SET-UP CLEARANCE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q201=" & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo)  & ";DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q206=" & e.FeedRate.ToString(feedrateFormat, CultureInfo) & ";FEED RATE FOR PLNGNG ~")
		NcCode.NewLine()
		If noPeck Or chipBreak
			NcCode.NewString("  Q202=" & (e.MaterialSurface - e.FinishDepth).ToString(positionFormat, CultureInfo) & ";PLUNGING DEPTH ~")
		Else
			NcCode.NewString("  Q202=" & e.PeckAmount & ";PLUNGING DEPTH ~")
		End If
		NcCode.NewLine()
		NcCode.NewString("  Q203=" & e.MaterialSurface.ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		If e.RetractToIntialRapid
			NcCode.NewString("  Q204=" & (e.IntialRapid - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE ~")
		Else
			NcCode.NewString("  Q204=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE ~")
		End If
		NcCode.NewLine()
		NcCode.NewString("  Q212=0;DECREMENT ~")
		NcCode.NewLine()
		NcCode.NewString("  Q205=0;MIN. INFEED DEPTH ~")
		NcCode.NewLine()
		'Q258 = 2 * effective pitch
		NcCode.NewString("  Q258=" & (2 * e.FeedRate/e.SpindleSpeed).ToString(feedrateFormat, CultureInfo) & ";UPPER ADVANCED STOP DISTANCE ~")
		NcCode.NewLine()
		'Q259 = 2 * effective pitch
		NcCode.NewString("  Q259=" & (2 * e.FeedRate/e.SpindleSpeed).ToString(feedrateFormat, CultureInfo) & ";LOWER ADVANCED STOP DISTANCE ~")
		NcCode.NewLine()
		If chipBreak
			NcCode.NewString("  Q257=" & e.PeckAmount.ToString(positionFormat, CultureInfo) & ";DEPTH FOR CHIP BRKNG ~")
		Else
			NcCode.NewString("  Q257=0;DEPTH FOR CHIP BRKNG ~")		
		End If
		NcCode.NewLine()
		'Retract by the effective pitch for chip break
		NcCode.NewString("  Q256=" & (e.FeedRate/e.SpindleSpeed).ToString(feedrateFormat, CultureInfo) & ";DIST. FOR CHIP BRKNG ~")
		NcCode.NewLine()
		NcCode.NewString("  Q211=" & e.Dwell.ToString(positionFormat, CultureInfo) & ";DWELL TIME AT DEPTH")
		NcCode.NewLine()
		
		Me.DrillNewCycle(e, afterToolChange)
		
	End Sub

	''' <summary>
	''' Output code for Cycle 206
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle206(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)
	
	    NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 206 TAPPING NEW ~")
		NcCode.NewLine()
		NcCode.NewString("  Q200=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q201=" & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo)  & ";DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q206=" & (e.Pitch * e.SpindleSpeed).ToString(positionFormat, CultureInfo) & ";FEED RATE FOR PLNGNG ~") 
		NcCode.NewLine()
		NcCode.NewString("  Q211=" & e.Dwell.ToString(positionFormat, CultureInfo) & ";DWELL TIME AT DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q203=" & e.MaterialSurface.ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		If e.RetractToIntialRapid
			NcCode.NewString("  Q204=" & (e.IntialRapid - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE")
		Else
			NcCode.NewString("  Q204=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE")
		End If
		NcCode.NewLine()
		
		Me.DrillNewCycle(e, afterToolChange)
		
 	End Sub

	''' <summary>
	''' Output code for Cycle 17
	''' </summary>
	''' <param name="e">The <see cref="DrillFirstHoleAfterToolChangeEventArgs"/> arguments</param>
	''' <param name="afterToolChange">A value indicating if the hole is the first after a tool change or in operation. <c>True</c> if first after tool change, <c>False</c> if first after operation</param>
	Private Sub Cycle207(ByVal e As DrillFirstHoleAfterToolChangeEventArgs, ByVal afterToolChange As Boolean)
	
	    NcCode.LineNumber()
		NcCode.NewString("CYCL DEF 207 RIGID TAPPING NEW ~")
		NcCode.NewLine()
		NcCode.NewString("  Q200=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		NcCode.NewString("  Q201=" & (e.FinishDepth - e.MaterialSurface).ToString(positionFormat, CultureInfo)  & ";DEPTH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q239=" & e.Pitch.ToString(positionFormat, CultureInfo) & ";PITCH ~")
		NcCode.NewLine()
		NcCode.NewString("  Q203=" & e.MaterialSurface.ToString(positionFormat, CultureInfo) & ";SURFACE COORDINATE ~")
		NcCode.NewLine()
		If e.RetractToIntialRapid
			NcCode.NewString("  Q204=" & (e.IntialRapid - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE")
		Else
			NcCode.NewString("  Q204=" & (e.FeedFrom - e.MaterialSurface).ToString(positionFormat, CultureInfo) & ";2ND SET-UP CLEARANCE")
		End If
		NcCode.NewLine()
		
		Me.DrillNewCycle(e, afterToolChange)
		
End Sub

#End Region

#Region "User Congigured Input"

    ''' <summary>
    ''' A String varable to store the program number, this is available through out the
    ''' Machine Definition
    ''' </summary>
    Private programNumber As String

    ''' <summary>
    ''' When the user clicks the 'User Configured Input' button on the NC Code tab in the Part Manager,
    ''' this sub procedure is called. Here we display the dialog box where the user can input inpositionFormation
	''' that can be used in the Machine Definition.
    ''' </summary>
    Private Sub UserConfiguredInput(ByVal e As UserConfiguredInputEventArgs) Implements IMachineDefinition_2_1.UserConfiguredInput

		UserConfiguredInputDialog.Display()
        programNumber = UserConfiguredInputDialog.GetInputBoxValue("Program Number")
        NcCode.GenerateCode()

    End Sub

#End Region

#Region "Properties"

    ''' <summary>
    ''' Gets the Nc code to be output for a clockwise spindle start.
    ''' </summary>
    Private ReadOnly Property CWSpindleStart() As String Implements IMachineDefinition_2_1.CWSpindleStart
        Get
            Return "M3"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for a counter clockwise spindle start.
    ''' </summary>
    Private ReadOnly Property CCWSpindleStart() As String Implements IMachineDefinition_2_1.CCWSpindleStart
        Get
            Return "M4"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for flood coolant.
    ''' </summary>
    Private ReadOnly Property CoolantFlood() As String Implements IMachineDefinition_2_1.CoolantFlood
        Get
            Return _configuration.FloodCoolant
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant through spindle.
    ''' </summary>
    Private ReadOnly Property CoolantThroughSpindle() As String Implements IMachineDefinition_2_1.CoolantThroughSpindle
        Get
            Return _configuration.ThroughSpindleCoolant
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant through the tool.
    ''' </summary>
    Private ReadOnly Property CoolantThroughTool() As String Implements IMachineDefinition_2_1.CoolantThroughTool
        Get
            Return _configuration.ThroughToolCoolant
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for mist coolant.
    ''' </summary>
    Private ReadOnly Property CoolantMist() As String Implements IMachineDefinition_2_1.CoolantMist
        Get
            Return _configuration.MistCoolant
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for an air blast.
    ''' </summary>
    Private ReadOnly Property CoolantAirBlast() As String Implements IMachineDefinition_2_1.CoolantAirBlast
        Get
            Return _configuration.AirBlastCoolant
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant off.
    ''' </summary>
    Private ReadOnly Property CoolantNone() As String Implements IMachineDefinition_2_1.CoolantNone
        Get
            Return "M9"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for custom coolant.
    ''' </summary>
    Private ReadOnly Property CoolantCustom() As String Implements IMachineDefinition_2_1.CoolantCustom
        Get
            Return ""
        End Get
    End Property

#End Region

#Region "Modals"

    ''' <summary>
    ''' This event handler is invoked to sets up modal words
    ''' </summary>
    ''' <param name="e">Provides data for the modals event</param>
    Private Sub Modals(ByVal e As ModalEventArgs) Implements IMachineDefinition_2_1.Modals

		'Add Absolute Modal Dimentioned Words to 'AbsoluteDimensionedWords'.	
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("X"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Y"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Z"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("F"))

    End Sub

#End Region

#Region "Miscellaneous"

    ''' <summary>
    ''' Outputs the Z axis retract to the Tool change position
    ''' </summary>
	''' <param name="metric">A value indicating if this Part units are metric</param>
	Private Sub ZAxisRetract(ByVal metric As Boolean)
		
		Dim zAxisPosition As String
		If metric
			zAxisPosition =  _configuration.ZAxisToolChangePosition.ToString(positionFormat, CultureInfo)
		Else
			zAxisPosition = (_configuration.ZAxisToolChangePosition / 25.4).ToString(positionFormat, CultureInfo)
		End If

		If Not _configuration.ToolChangeRetractMethod = ToolChangeRetractMethod.DoNothing Then
			NcCode.LineNumber()
		End If
		If _configuration.ToolChangeRetractMethod = ToolChangeRetractMethod.ToolCall0 Then
        	NcCode.NewString("TOOL CALL 0")
			NcCode.NewLine()
        	NcCode.LineNumber()
		    NcCode.NewString("L Z" & zAxisPosition & " R0 " & maxFeedrate)
			If _configuration.OutputSTOPBeforeTOOLCALL Then
			    NcCode.NewLine()
				NcCode.LineNumber()
				NcCode.NewString("STOP M5")
			End If
		Else If _configuration.ToolChangeRetractMethod = ToolChangeRetractMethod.StopM25 Then
        	NcCode.NewString("STOP M25")
		Else If _configuration.ToolChangeRetractMethod = ToolChangeRetractMethod.ZM91 Then
		    NcCode.NewString("L Z" & zAxisPosition & " R0 " & maxFeedrate & " M91")
			If _configuration.OutputSTOPBeforeTOOLCALL Then
			    NcCode.NewLine()
				NcCode.LineNumber()
				NcCode.NewString("STOP M5")
			End If
		Else If _configuration.ToolChangeRetractMethod = ToolChangeRetractMethod.ZM92 Then
		    NcCode.NewString("L Z" & zAxisPosition & " R0 " & maxFeedrate & " M92")
			If _configuration.OutputSTOPBeforeTOOLCALL Then
			    NcCode.NewLine()
				NcCode.LineNumber()
				NcCode.NewString("STOP M5")
			End If
		Else If _configuration.ToolChangeRetractMethod = ToolChangeRetractMethod.PrgCall Then
		    NcCode.NewString("CALL PGM " & _configuration.ToolRetractLabel)
		End If
		If Not _configuration.ToolChangeRetractMethod = ToolChangeRetractMethod.DoNothing Then
        	NcCode.NewLine()
		End If

	End Sub

#End Region

End Class

#Region "Configuration"

''' <summary>
''' Class to manage the configuration of this Machine Definition.
''' </summary>
Public Class Configuration

    Private _m6RequiredForATC As Boolean
    Private _preselectNextTool As Boolean
    Private _toolChangeRetractMethod As ToolChangeRetractMethod
    Private _toolRetractLabel As String
    Private _outputM01ForATC As M01Type
    Private _outputBlankForm As Boolean
    Private _toolList As Boolean
    Private _structuredBlockAtATC As Boolean
    Private _drilling As DrillingCycle
    Private _peckDrill As PeckDrillCycle
    Private _chipBreaker As ChipBreakerCycle
    Private _tapping As TappingCycle
    Private _rigidTapping As RigidTappingCycle
    Private _floodCoolant As String
    Private _throughSpindleCoolant As String
    Private _throughToolCoolant As String
    Private _mistCoolant As String
    Private _airBlastCoolant As String
    Private _customCoolant As String
    Private _userFileNameAsProgramName As Boolean
    Private _useM13 As Boolean
    Private _fMax As Boolean
    Private _maxFeedrate As Double
    Private _zAxisToolChangePosition As Double
    Private _outputSTOPBeforeTOOLCALL As Boolean
    Private _yAxisUnloadMethod As YAxisUnloadMethod
    Private _yAxisUnloadPosition As Double
	Private _outputCycleTime As Boolean
	Private _chipToChipAtcTime As Double
	Private _rapidTraverseRate As Double
	Private _rapidPositionAccelerationDeceleration As Double
	Private _outputToolDef As Boolean
	Private _programEndMCode As ProgramEndMCode
	Private _numberDecimalPlacesPositionMetric As Integer
	Private _numberDecimalPlacesPositionInch As Integer
	Private _numberDecimalPlacesFeedrateMetric As Integer
	Private _numberDecimalPlacesFeedrateInch As Integer
    Private _nonModalFeedrate As Boolean
    Private _outputToolOffsetValue As Boolean

	
    ''' <summary>
    ''' Gets or sets a value indicating if an M6 should be output for an Automatic Tool Change.
    ''' </summary>
    <System.ComponentModel.Description("Should an M6 be output for an Automatic Tool Change."), _
    System.ComponentModel.DisplayName("M6 for ATC"), _
    System.ComponentModel.Category("Tool Change")> _
    Public Property M6RequiredForATC() As Boolean
        Get
            Return _m6RequiredForATC
        End Get
        Set(ByVal value As Boolean)
            _m6RequiredForATC = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if the next Tool should be pre selected.
    ''' </summary>
    <System.ComponentModel.Description("Should the next Tool be pre selected."), _
    System.ComponentModel.DisplayName("Pre select next Tool"), _
    System.ComponentModel.Category("Tool Change")> _
    Public Property PreselectNextTool() As Boolean
        Get
            Return _preselectNextTool
        End Get
        Set(ByVal value As Boolean)
            _preselectNextTool = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the Method used to return the Z axis to the tool change position.
    ''' </summary>
    <System.ComponentModel.Description("The Method used to return the Z axis to the tool change position."), _
    System.ComponentModel.DisplayName("Z axis return method"), _
    System.ComponentModel.Category("Tool Change")> _
    Public Property ToolChangeRetractMethod() As ToolChangeRetractMethod
        Get
            Return _toolChangeRetractMethod
        End Get
        Set(ByVal value As ToolChangeRetractMethod)
            _toolChangeRetractMethod = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the Z axis position the Tool change takes place.
    ''' </summary>
    <System.ComponentModel.Description("The Z axis position in METRIC the Tool change takes place"), _
    System.ComponentModel.DisplayName("Z axis position"), _
    System.ComponentModel.Category("Tool Change")> _
    Public Property ZAxisToolChangePosition() As Double
        Get
            Return _zAxisToolChangePosition
        End Get
        Set(ByVal value As Double)
            _zAxisToolChangePosition = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the program name/number called for retracting the Z axis to the Tool change position.
    ''' </summary>
    <System.ComponentModel.Description("Program name/number called for retracting the Z axis to the Tool change position."), _
    System.ComponentModel.DisplayName("Program name/number for Z axis retract"), _
    System.ComponentModel.Category("Tool Change")> _
    Public Property ToolRetractLabel() As String
        Get
            Return _toolRetractLabel
        End Get
        Set(ByVal value As String)
            _toolRetractLabel = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if and where an M1 is output for an ATC.
    ''' </summary>
    <System.ComponentModel.Description("Specifies if and where an M1 is output for an ATC"), _
 System.ComponentModel.DisplayName("Output M1 for ATC"), _
 System.ComponentModel.Category("Tool Change")> _
 Public Property OutputM01ForATC() As M01Type
        Get
            Return _outputM01ForATC
        End Get
        Set(ByVal value As M01Type)
            _outputM01ForATC = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if a STOP should be output before a TOOL CALL.
    ''' </summary>
    <System.ComponentModel.Description("Specifies if a STOP should be output before a TOOL CALL"), _
    System.ComponentModel.DisplayName("Stop before TOOL CALL"), _
    System.ComponentModel.Category("Tool Change")> _
    Public Property OutputSTOPBeforeTOOLCALL() As Boolean
        Get
            Return _outputSTOPBeforeTOOLCALL
        End Get
        Set(ByVal value As Boolean)
            _outputSTOPBeforeTOOLCALL = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if a Blank Form should be output.
    ''' </summary>
    <System.ComponentModel.Description("Should a Blank Form be output?"), _
    System.ComponentModel.DisplayName("Output Blank Form")> _
    Public Property OutputBlankForm() As Boolean
        Get
            Return _outputBlankForm
        End Get
        Set(ByVal value As Boolean)
            _outputBlankForm = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if a Tooling list should be output.
    ''' </summary>
    <System.ComponentModel.Description("Should a Tooling list and Tool Name on the TOOL CALL line be output? Only choose if your control supports comments"), _
 System.ComponentModel.DisplayName("Output a Tooling list")> _
 Public Property ToolList() As Boolean
        Get
            Return _toolList
        End Get
        Set(ByVal value As Boolean)
            _toolList = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if a Structured Block should be output at the tool change call.
    ''' </summary>
    <System.ComponentModel.Description("Should a Structured Block should be output at the tool change call?"), _
    System.ComponentModel.DisplayName("Structured Block at ATC")> _
    Public Property StructuredBlockAtATC() As Boolean
        Get
            Return _structuredBlockAtATC
        End Get
        Set(ByVal value As Boolean)
            _structuredBlockAtATC = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating which cycle is to be used when the type of drilling is Drilling
    ''' </summary>
    <System.ComponentModel.Description("Drilling cycle to be used when the type of drilling is Drilling."), _
    System.ComponentModel.DisplayName("Cycle for Drilling"), _
    System.ComponentModel.Category("Drilling Cycles")> _
    Public Property DrillingCycle() As DrillingCycle
        Get
            Return _drilling
        End Get
        Set(ByVal value As DrillingCycle)
            _drilling = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating which cycle is to be used when the type of drilling is Peck Drill
    ''' </summary>
    <System.ComponentModel.Description("Drilling cycle to be used when the type of drilling is Peck Drill."), _
    System.ComponentModel.DisplayName("Cycle for Peck Drill"), _
    System.ComponentModel.Category("Drilling Cycles")> _
    Public Property PeckDrillCycle() As PeckDrillCycle
        Get
            Return _peckDrill
        End Get
        Set(ByVal value As PeckDrillCycle)
            _peckDrill = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating which cycle is to be used when the type of drilling is Chip Breaker
    ''' </summary>
    <System.ComponentModel.Description("Drilling cycle to be used when the type of drilling is Chip Breaker."), _
    System.ComponentModel.DisplayName("Cycle for Chip Breaker"), _
    System.ComponentModel.Category("Drilling Cycles")> _
    Public Property ChipBreakerCycle() As ChipBreakerCycle
        Get
            Return _chipBreaker
        End Get
        Set(ByVal value As ChipBreakerCycle)
            _chipBreaker = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating which cycle is to be used when the type of drilling is Tapping
    ''' </summary>
    <System.ComponentModel.Description("Drilling cycle to be used when the type of drilling is Tapping."), _
    System.ComponentModel.DisplayName("Cycle for Tapping"), _
    System.ComponentModel.Category("Drilling Cycles")> _
    Public Property TappingCycle() As TappingCycle
        Get
            Return _tapping
        End Get
        Set(ByVal value As TappingCycle)
            _tapping = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating which cycle is to be used when the type of drilling is Rigid Tapping
    ''' </summary>
    <System.ComponentModel.Description("Drilling cycle to be used when the type of drilling is Rigid Tapping."), _
    System.ComponentModel.DisplayName("Cycle for Rigid Tapping"), _
    System.ComponentModel.Category("Drilling Cycles")> _
    Public Property RigidTappingCycle() As RigidTappingCycle
        Get
            Return _rigidTapping
        End Get
        Set(ByVal value As RigidTappingCycle)
            _rigidTapping = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the M code that turns on the Flood Coolant.
    ''' </summary>
    <System.ComponentModel.Description("The M code that turns on the Flood Coolant."), _
    System.ComponentModel.DisplayName("Flood Coolant M code"), _
    System.ComponentModel.Category("Coolant")> _
    Public Property FloodCoolant() As String
        Get
            Return _floodCoolant
        End Get
        Set(ByVal value As String)
            _floodCoolant = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the M code that turns on the Through Spindle Coolant.
    ''' </summary>
    <System.ComponentModel.Description("The M code that turns on the Through Spindle Coolant."), _
    System.ComponentModel.DisplayName("Through Spindle Coolant M code"), _
    System.ComponentModel.Category("Coolant")> _
    Public Property ThroughSpindleCoolant() As String
        Get
            Return _throughSpindleCoolant
        End Get
        Set(ByVal value As String)
            _throughSpindleCoolant = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the M code that turns on the Through Tool Coolant.
    ''' </summary>
    <System.ComponentModel.Description("The M code that turns on the Through Tool Coolant."), _
    System.ComponentModel.DisplayName("Through Tool Coolant M code"), _
    System.ComponentModel.Category("Coolant")> _
    Public Property ThroughToolCoolant() As String
        Get
            Return _throughToolCoolant
        End Get
        Set(ByVal value As String)
            _throughToolCoolant = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the M code that turns on the Mist Coolant.
    ''' </summary>
    <System.ComponentModel.Description("The M code that turns on the Mist Coolant."), _
    System.ComponentModel.DisplayName("Mist Coolant M code"), _
    System.ComponentModel.Category("Coolant")> _
    Public Property MistCoolant() As String
        Get
            Return _mistCoolant
        End Get
        Set(ByVal value As String)
            _mistCoolant = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the M code that turns on the AirBlast Coolant.
    ''' </summary>
    <System.ComponentModel.Description("The M code that turns on the AirBlast Coolant."), _
    System.ComponentModel.DisplayName("AirBlast Coolant M code"), _
    System.ComponentModel.Category("Coolant")> _
    Public Property AirBlastCoolant() As String
        Get
            Return _airBlastCoolant
        End Get
        Set(ByVal value As String)
            _airBlastCoolant = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the M code that turns on the Custom Coolant.
    ''' </summary>
    <System.ComponentModel.Description("The M code that turns on the Custom Coolant."), _
    System.ComponentModel.DisplayName("Custom Coolant M code"), _
    System.ComponentModel.Category("Coolant")> _
    Public Property CustomCoolant() As String
        Get
            Return _customCoolant
        End Get
        Set(ByVal value As String)
            _customCoolant = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if M13/M14 should be used when Flood coolant is being used.
    ''' </summary>
    <System.ComponentModel.Description("Use M13/M14 when Food coolant is being used."), _
    System.ComponentModel.DisplayName("Use M13"), _
    System.ComponentModel.Category("Coolant")> _
    Public Property UseM13() As Boolean
        Get
            Return _useM13
        End Get
        Set(ByVal value As Boolean)
            _useM13 = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if the file name or a number be used as the program name.
    ''' </summary>
    <System.ComponentModel.Description("Should the file name or a number be used as the program name. True for filename, False for number."), _
    System.ComponentModel.DisplayName("Program Name")> _
    Public Property UserFileNameAsProgramName() As Boolean
        Get
            Return _userFileNameAsProgramName
        End Get
        Set(ByVal value As Boolean)
            _userFileNameAsProgramName = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if the control supports FMAX.
    ''' </summary>
    <System.ComponentModel.Description("Does the control support FMAX? If set to False then Maximum Feedrate property is used."), _
    System.ComponentModel.DisplayName("FMAX"), _
    System.ComponentModel.Category("Rapids")> _
    Public Property FMax() As Boolean
        Get
            Return _fMax
        End Get
        Set(ByVal value As Boolean)
            _fMax = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the maximum feedrate of the control.
    ''' </summary>
    <System.ComponentModel.Description("Maximum feedrate of the control. Used when FMAX property is set to False"), _
    System.ComponentModel.DisplayName("Maximum Feedrate"), _
    System.ComponentModel.Category("Rapids")> _
    Public Property MaxFeedrate() As Double
        Get
            Return _maxFeedrate
        End Get
        Set(ByVal value As Double)
            _maxFeedrate = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the method for positioning the Y axis at the unload position.
    ''' </summary>
    <System.ComponentModel.Description("The method for positioning the Y axis at the unload position"), _
    System.ComponentModel.DisplayName("Y Axis Unload"), _
    System.ComponentModel.Category("Unload")> _
    Public Property YAxisUnloadMethod() As YAxisUnloadMethod
        Get
            Return _yAxisUnloadMethod
        End Get
        Set(ByVal value As YAxisUnloadMethod)
            _yAxisUnloadMethod = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the position from the reference point that the Y axis is positioned at the end of the program.
    ''' </summary>
    <System.ComponentModel.Description("The position in METRIC from the reference point that the Y axis is positioned at the end of the program."), _
    System.ComponentModel.DisplayName("Y axis unload position"), _
    System.ComponentModel.Category("Unload")> _
    Public Property YAxisUnloadPosition() As Double
        Get
            Return _yAxisUnloadPosition
        End Get
        Set(ByVal value As Double)
            _yAxisUnloadPosition = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets a value indicating if the cycle time should be output in a comment.
	''' </summary>
	<System.ComponentModel.Description("Should the cycle time be output?"), _
	System.ComponentModel.DisplayName("Output Cycle Time"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property OutputCycleTime() As Boolean
        Get
            Return _outputCycleTime
        End Get
        Set
            _outputCycleTime = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the chip to chip tool change time in seconds.
	''' </summary>
	<System.ComponentModel.Description("The chip to chip tool change time in seconds"), _
	System.ComponentModel.DisplayName("Chip to chip tool change time"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property ChipToChipAtcTime() As Double
        Get
            Return _chipToChipAtcTime
        End Get
        Set
            _chipToChipAtcTime = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the rapid traverse rate in millimetres/minute.
	''' </summary>
	<System.ComponentModel.Description("The rapid traverse rate in millimetres/minute. This value is converted to imperial for Parts with Inch units."), _
	System.ComponentModel.DisplayName("Rapid traverse rate"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property RapidTraverseRate() As Double
        Get
            Return _rapidTraverseRate
        End Get
        Set
            _rapidTraverseRate = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the time for a each Rapid move that should be added to the cycle time due to Acceleration/Deceleration.
	''' </summary>
	<System.ComponentModel.Description("The time for a each Rapid move that should be added to the cycle time due to Acceleration/Deceleration"), _
	System.ComponentModel.DisplayName("Rapid Acceleration/Deceleration"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property RapidPositionAccelerationDeceleration() As Double
        Get
            Return _rapidPositionAccelerationDeceleration

        End Get
        Set
            _rapidPositionAccelerationDeceleration = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets a value indicating if the Tool defintions should be output at the head of the program.
	''' </summary>
	<System.ComponentModel.Description("Should the Tool Defintions be output?"), _
	System.ComponentModel.DisplayName("Tool Defintions")> _
	Public Property OutputToolDef() As Boolean
        Get
            Return _outputToolDef
        End Get
        Set
            _outputToolDef = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets a value indicating if the Tool defintions should be output at the head of the program.
	''' </summary>
	<System.ComponentModel.Description("The M code to use for the program end."), _
	System.ComponentModel.DisplayName("Program End")> _
	Public Property ProgramEndMCode() As ProgramEndMCode
        Get
            Return _programEndMCode
        End Get
        Set
            _programEndMCode = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets the number of decimal places to display for X, Y, and X positions after the decimal separator for a metric Part.
	''' </summary>
	<System.ComponentModel.Description("The number of decimal places to display for X, Y, and X positions after the decimal separator for a metric Part"), _
	System.ComponentModel.DisplayName("Metric Part Position Decimal Places")> _
	Public Property NumberDecimalPlacesPositionMetric() As Integer
        Get
            Return _numberDecimalPlacesPositionMetric
        End Get
        Set
            _numberDecimalPlacesPositionMetric = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets the number of decimal places to display for X, Y, and X positions after the decimal separator for an Inch Part.
	''' </summary>
	<System.ComponentModel.Description("The number of decimal places to display for X, Y, and X positions after the decimal separator for an inch Part"), _
	System.ComponentModel.DisplayName("Inch Part Position Decimal Places")> _
	Public Property NumberDecimalPlacesPositionInch() As Integer
        Get
            Return _numberDecimalPlacesPositionInch
        End Get
        Set
            _numberDecimalPlacesPositionInch = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets the number of decimal places to display for feedrates after the decimal separator for a metric Part.
	''' </summary>
	<System.ComponentModel.Description("The number of decimal places to display for feedrates after the decimal separator for a metric Part"), _
	System.ComponentModel.DisplayName("Metric Part Feedrate Decimal Places")> _
	Public Property NumberDecimalPlacesFeedrateMetric() As Integer
        Get
            Return _numberDecimalPlacesFeedrateMetric
        End Get
        Set
            _numberDecimalPlacesFeedrateMetric = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets the number of decimal places to display for feedrates after the decimal separator for an Inch Part.
	''' </summary>
	<System.ComponentModel.Description("The number of decimal places to display for feedrates after the decimal separator for an inch Part"), _
	System.ComponentModel.DisplayName("Inch Part Feedrate Decimal Places")> _
	Public Property NumberDecimalPlacesFeedrateInch() As Integer
        Get
            Return _numberDecimalPlacesFeedrateInch
        End Get
        Set
            _numberDecimalPlacesFeedrateInch = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets a value indicating if the feedrate should be output even if the same as the previous value. Also an M will be appended to the line
	''' </summary>
	<System.ComponentModel.Description("Should the feedrate should be output even if the same as the previous value? Also an M will be appended to the line."), _
	System.ComponentModel.DisplayName("Feedrate Non Modal")> _
	Public Property NonModalFeedrate() As Boolean
        Get
            Return _nonModalFeedrate
        End Get
        Set
            _nonModalFeedrate = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating if the Offset value from the Tool Manger is used in the TOOL DEF
    ''' </summary>
    <System.ComponentModel.Description("Use Offset value from Tool Manager for length in TOOL DEF"), _
    System.ComponentModel.DisplayName("Use Offet value")> _
    Public Property OutputToolOffsetValue() As Boolean
        Get
            Return _outputToolOffsetValue
        End Get
        Set(ByVal value As Boolean)
            _outputToolOffsetValue = value
        End Set
    End Property

End Class

''' <summary>
''' The Method used to return the Z axis to the tool change position.
''' </summary>
Public Enum ToolChangeRetractMethod
	
	''' <summary>
	''' TOOL CALL 0
	''' </summary>		
	ToolCall0
	
	''' <summary>
	''' Stop and M25
	''' </summary>		
	StopM25
	
	''' <summary>
	''' Z position and M91
	''' </summary>
	ZM91
	
	''' <summary>
	''' Z position and M92
	''' </summary>
	ZM92
	
	''' <summary>
    ''' A program is called to return the Z axis to the tool change position
	''' </summary>
	PrgCall
		
	''' <summary>
    ''' Do nothing
	''' </summary>
	DoNothing
	
End Enum

''' <summary>
''' The Method used to position the Y axis at the unload position.
''' </summary>
Public Enum YAxisUnloadMethod
	
	''' <summary>
	''' TOOL CALL 0
	''' </summary>		
	ToolCall0
	
	''' <summary>
	''' Y position and M91
	''' </summary>
	ZM91
			
	''' <summary>
	''' Y position and M92
	''' </summary>
	ZM92
	
	''' <summary>
    ''' Do nothing
	''' </summary>
	DoNothing
		
End Enum

''' <summary>
''' The position of M01.
''' </summary>
Public Enum M01Type

	''' <summary>
	''' Do not output M01.
	''' </summary>
	DoNotOutput

	''' <summary>
	''' Output M01 before the ATC.
	''' </summary>
	OutputBeforeATC
	
	''' <summary>
	''' Output M01 after the ATC.
	''' </summary>
	OutputAfterATC
	
End Enum

''' <summary>
''' The cycle to use for the Drilling command as chosen by the user from the drop down box while drilling.
''' </summary>
Public Enum DrillingCycle

	''' <summary>
	''' Cycle 1 - Pecking
	''' </summary>
	Cycle1

	''' <summary>
	''' Cycle 200 - Drilling
	''' </summary>
	Cycle200
	
	''' <summary>
	''' Cycle 203 - Universal Drilling
	''' </summary>
	Cycle203
	
End Enum

''' <summary>
''' The cycle to use for the Peck Drill command as chosen by the user from the drop down box while drilling.
''' </summary>
Public Enum PeckDrillCycle

	''' <summary>
	''' Cycle 1 - Pecking
	''' </summary>
	Cycle1

	''' <summary>
	''' Cycle 200 - Drilling
	''' </summary>
	Cycle200
	
	''' <summary>
	''' Cycle 203 - Universal Drilling
	''' </summary>
	Cycle203
		
	''' <summary>
	''' Cycle 205 - Universal Pecking
	''' </summary>
	Cycle205
	
End Enum

''' <summary>
''' The cycle to use for the Chip Breaker command as chosen by the user from the drop down box while drilling.
''' </summary>
Public Enum ChipBreakerCycle

	''' <summary>
	''' Cycle 203 - Universal Drilling
	''' </summary>
	Cycle203
		
	''' <summary>
	''' Cycle 205 - Universal Pecking
	''' </summary>
	Cycle205
	
End Enum

''' <summary>
''' The cycle to use for the Tapping command as chosen by the user from the drop down box while drilling.
''' </summary>
Public Enum TappingCycle

	''' <summary>
	''' Cycle 2 - Tapping
	''' </summary>
	Cycle2
		
	''' <summary>
	''' Cycle 206 - New Tapping
	''' </summary>
	Cycle206
	
End Enum

''' <summary>
''' The cycle to use for the Rigid Tapping command as chosen by the user from the drop down box while drilling.
''' </summary>
Public Enum RigidTappingCycle

	''' <summary>
	''' Cycle 207 - New Tapping
	''' </summary>
	Cycle17
			
	''' <summary>
	''' Cycle 209- Tapping With Chip Breaker
	''' </summary>
	Cycle207
	
End Enum

''' <summary>
''' The M code to use for the end of the program
''' </summary>
Public Enum ProgramEndMCode

	''' <summary>
	''' Use M30
	''' </summary>
	M30
			
	''' <summary>
	''' Use M02
	''' </summary>
	M02
		''' <summary>
	''' Do not output M01.
	''' </summary>
	DoNotOutput
	
End Enum

#End Region

